package cc.fascinated.backend.model.account;

import cc.fascinated.backend.common.DateUtils;
import cc.fascinated.backend.model.token.ScoreSaberAccountToken;
import io.micrometer.common.lang.NonNull;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;
import lombok.SneakyThrows;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.mapping.Document;

import java.util.Date;

/**
 * An account for ScoreSaber.
 */
@Document
@AllArgsConstructor
@Getter @Setter
public class Account {
    /**
     * The id for this ScoreSaber account.
     */
    @Id @NonNull
    private final String id;

    /**
     * The name for this account.
     */
    private String name;

    /**
     * The profile picture for this account.
     */
    private String profilePicture;

    /**
     * The bio for this account.
     */
    private Bio bio;

    /**
     * The country for this account.
     */
    private String country;

    /**
     * The PP for this account.
     */
    private double performancePoints;

    /**
     * The amount of raw pp to get 1 global performance point.
     */
    private double rawPerGlobalPerformancePoints;

    /**
     * The rank for this account.
     */
    private int rank;

    /**
     * The country rank for this account.
     */
    private int countryRank;

    /**
     * The role for this account.
     * todo: make this an enum
     */
    private String role;

    /**
     * The badges for this account.
     */
    private Badge[] badges;

    /**
     * The history of the rank for this account.
     */
    private int[] rankHistory;

    /**
     * The permissions for this account.
     */
    private int permission;

    /**
     * The banned status for this account.
     */
    private boolean banned;

    /**
     * The inactive status for this account.
     */
    private boolean inactive;

    /**
     * The score stats for this account.
     */
    private ScoreStats scoreStats;

    /**
     * When the account joined ScoreSaber.
     */
    private Date firstSeen;

    /**
     * Gets the account from the given token.
     *
     * @param token The token.
     * @return The account.
     */
    @SneakyThrows
    public static Account fromToken(ScoreSaberAccountToken token) {
        int[] rankHistory = new int[token.getHistories().split(",").length];
        for (int i = 0; i < rankHistory.length; i++) {
            rankHistory[i] = Integer.parseInt(token.getHistories().split(",")[i]);
        }

        // Convert the token to an account.
        return new Account(
                token.getId(),
                token.getName(),
                token.getProfilePicture(),
                Bio.fromRaw(token.getBio()),
                token.getCountry(),
                token.getPp(),
                -1,
                token.getRank(),
                token.getCountryRank(),
                token.getRole(),
                token.getBadges(),
                rankHistory,
                token.getPermissions(),
                token.isBanned(),
                token.isInactive(),
                token.getScoreStats(),
                DateUtils.getDateFromString(token.getFirstSeen())
        );
    }

    /**
     * The bio for this account.
     */
    @AllArgsConstructor @Getter
    public static class Bio {
        /**
         * The raw bio.
         */
        private String[] raw;

        /**
         * The clean bio with no HTML tags.
         */
        private String[] clean;

        /**
         * Gets the bio from the raw string.
         *
         * @param raw The raw bio.
         * @return The bio.
         */
        public static Bio fromRaw(String raw) {
            if (raw == null || raw.isEmpty()) {
                return null;
            }
            return new Bio(
                    raw.split("\n"),
                    raw.replaceAll("<[^>]*>", "").split("\n")
            );
        }
    }

    /**
     * The badge for this account.
     */
    @AllArgsConstructor @Getter
    public static class Badge {
        /**
         * The image for this badge.
         */
        private String image;

        /**
         * The description for this badge.
         */
        private String description;
    }

    /**
     * The score stats for this account.
     */
    @AllArgsConstructor @Getter
    public static class ScoreStats {
        /**
         * The total score for this account.
         */
        private long totalScore;

        /**
         * The total ranked score for this account.
         */
        private long totalRankedScore;

        /**
         * The average ranked accuracy for this account.
         */
        private double averageRankedAccuracy;

        /**
         * The total play count for this account.
         */
        private int totalPlayCount;

        /**
         * The ranked play count for this account.
         */
        private int rankedPlayCount;

        /**
         * The replays watched for this account.
         */
        private int replaysWatched;
    }
}
